/*
 * Fixed point support for the Ogg Vorbis audio decoding library
 *
 * Copyright (C) 2002 by Nicolas Pitre <nico@cam.org>
 *
 * You can use and/or redistribute this code and/or modify it under the
 * terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation. See the file COPYING for details.
 */

#ifndef FIXP_H
#define FIXP_H


#define FIXED_POINT
#ifdef FIXED_POINT

#define FIXP_FRACBITS	28

typedef int FIXP;

/*
 * This potentially make the code faster but with a certain accuracy penalty.
 */
//#define FIXP_SPEED

/* 
 * Rounding doesn't seem to make any visible difference on the final output,
 * we therefore neglect it for less CPU usage.
 */
#define TO_FIXP(s, x)	(FIXP)((x) * (1 << (s)))

//#define FIXP_CHK
#ifndef FIXP_CHK

#define TO_FIXP_CHK	TO_FIXP

#if defined(__arm__)

/* ARM optimized multiplication with proper rounding */
#define MUL(s, a, b)  ({ \
	int res, tmp; \
	asm (	"smull	%0, %1, %2, %3\n\t" \
		"movs	%0, %0, lsr %4\n\t" \
		"adc	%0, %0, %1, lsl %5" \
		: "=&r" (res), "=&r" (tmp) \
		: "%r" (a), "r" (b), \
		  "M" (s), "M"(32 - (s)) \
		: "cc"); \
	res; })

#else

/* Generic portable multiplication, need 64 bit data type support */
#define MUL(s, a, b)	(FIXP)(((ogg_int64_t)(a) * (b)) >> (s))

#endif

#else  /* FIXED_POINT_CHK */

/* This is for debugging only */

#include <stdio.h>

#define TO_FIXP_CHK(s, x) \
({ \
	if ((x) >= (1 << (31-(s))) || (x) < -(1 << (31-(s)))) \
		fprintf(stderr, "TO_FIXP(%d) ovf: %g [%s:%d]\n", \
			(s), (float)(x), __FUNCTION__, __LINE__); \
	if ((x) < 1.0/(1 << (s)) && (x) > -1.0/(1 << (s)) && (x) != 0.0) \
		fprintf(stderr, "TO_FIXP(%d) udf: %g [%s:%d]\n", \
			(s), (float)(x), __FUNCTION__, __LINE__); \
	TO_FIXP(s, x); \
})

#define MUL(s, a, b) \
({ \
	ogg_int64_t __x = ((ogg_int64_t)(a) * (b)) >> (s); \
	if (__x >= (1ll << 31) || __x < -(1ll << 31)) \
		fprintf(stderr, "MUL(%d) ovf: 0x%08x * 0x%08x (%s:%d)\n", \
			(s), (a), (b), __FUNCTION__, __LINE__); \
	(FIXP)__x; \
})

#endif  /* FIXED_POINT_CHK */

#else  /* FIXED_POINT */

typedef float FIXP;

#define TO_FIXP(s, x)	(x)
#define TO_FIXP_CHK	TO_FIXP
#define MUL(s, a, b)	((a) * (b))

#endif  /* FIXED_POINT */

#endif  /* FIXP_H */
